#include <Arduino.h>
/* Trafo-Tester
   Variation von https://www.youtube.com/watch?v=E1dopmxph2A
   mit OLed Display und ATtiny85
   Software Basis vom Batterietester von Andreas Spieß
   https://www.youtube.com/watch?v=mKPKQm0uPBQ
   Knackpunkt PCINT$ PinChange ISR Zähler Variable
   volatile int zaehler nur dann wird Variable geschrieben
   Spannungsteiler Versorgung 4:1 33k und 6,8k
*/

#include <TinyOzOLED.h>
#include <TinyWireM.h>

// ATtiny Pin 2 = ADC       ATtiny Pin 3 = Schalter für Fet 
// ATtiny Pin 5 = SDA       ATtiny Pin 6 = ZaehlerPin vom Kompaerator
// ATtiny Pin 7 = SCK       ATtiny Pin 8 = VCC (2.7-5.5V)

#define ADCVOLT A3  // ATtiny Pin 2
#define SchaltPin 4 // ATtiny Pin 3

// Spannungen am ADC bis 4V brauchen Spannungsteiler 4:1
const float FACTOR = 4.12; // Adjust for 4:1 Voltage divider and internal reference 330kOhm + 120kOhm

volatile int zaehler = 0; // Volatile ist wichtig damit Variable in ISR beschrieben wird

int emptyRaw;
int emptyOld;
double empty;
int emptySum;

ISR(PCINT0_vect)
{
  zaehler = zaehler + 1;
}

void initInterrupt(void)
{
  // pin change interrupt (example for D4)
  PCMSK |= bit(PCINT1); // want pin D1 / pin 6
  GIFR |= bit(PCIF);    // clear any outstanding interrupts
  GIMSK |= bit(PCIE);   // enable pin change interrupts
}

void lese_VCC()
{
  // wait till value stable
  do
  {
    emptyOld = emptyRaw;
    emptyRaw = analogRead(ADCVOLT);
  } while (abs(emptyRaw - emptyOld) > 3);

  // Average over 10 cycles
  emptySum = 0;
  for (int i = 0; i < 10; i++)
  {
    emptySum = emptySum + analogRead(ADCVOLT);
    delay(1);
  }
  empty = emptySum / 10230.0 * FACTOR;
}

void displayOLED()
{
  char tmp[10];
  if (empty < 2.90) //Wenn unter 2,9V Batt Spannung dann anderen Bildschirm anzeigen
  {
    OzOled.setCursorXY(4, 4); // Reihe 3, Zeile 4, für Mini-Display 64x32
    OzOled.printString("Batterie");
    OzOled.setCursorXY(4, 5);
    OzOled.printString("L E E R ");
    OzOled.setCursorXY(4, 6);
    dtostrf(empty, 3, 3, tmp);
    OzOled.printString(tmp);
    OzOled.setCursorXY(9, 6);
    OzOled.printString("V  ");
    OzOled.setCursorXY(4, 7);
    OzOled.printString("      ");
    OzOled.setCursorXY(9, 7);
    dtostrf(zaehler / 2, 3, 0, tmp);
    OzOled.printString(tmp);
  }
  else
  {
    dtostrf(zaehler / 2, 3, 0, tmp);
    OzOled.printBigNumber(tmp, 3, 4, 3);
  }
}

void setup()
{
  analogReference(INTERNAL); // use precise internal reference
  pinMode(ADCVOLT, INPUT);
  pinMode(SchaltPin, OUTPUT);

  OzOled.init();             // initialze OLED display
  OzOled.clearDisplay();     // clear the screen and set start position to top left corner
  OzOled.setNormalDisplay(); // Set display to Normal mode
  OzOled.sendCommand(0xA1);  // set Orientation
  OzOled.sendCommand(0xC8);
  OzOled.setCursorXY(4, 4);  // Reihe 4, Zeile 4, für Mini-Display 64x32
  OzOled.printString("Version");
  OzOled.setCursorXY(5, 5);
  OzOled.printString("3.0.2");
  OzOled.setCursorXY(4, 6);
  OzOled.printString("DL1MFK");
  OzOled.setCursorXY(4, 7);
  OzOled.printString("02/2023");
  delay(2000);
  digitalWrite(SchaltPin, LOW);
  initInterrupt();
  empty = 3.00; // damit keine Batt-Warnung am Anfang ausgegeben wird
}

void loop()
{
  zaehler = 0;
  digitalWrite(SchaltPin, HIGH);
  delay(500);
  displayOLED();
  digitalWrite(SchaltPin, LOW);
  lese_VCC();
  delay(150);
}